#!/usr/bin/env python
from binsearch import binsearch

# ignoring built-in python set.intersect()
# and the "in" keyword

# Naive brute-force search
def naiveIntersect(set1, set2):
    retset = set()
    for v in set1:
        for w in set2:
            if v == w:
                retset.add(v)
    return retset

# this will be drastically slower if set1
# is significantly larger than set2
def binarysearchIntersect(set1, set2):
    retset = set()
    print "(Native)"
    testset = sorted(set2)
    print "(End Native)"
    for v in set1:
        if binsearch(testset, v):
        #if v in testset:
            retset.add(v)
    return retset

#this should be relatively size agnostic since
# either emptying will terminate the loop
def minIntersect(set1,set2):
    retset = set()
    print "(Native)"
    set1 = sorted(set1) # we don't REALLY care how this is done.
    set2 = sorted(set2) # it should be basically n log n
                        # but it's C, so 2 orders of magnitude faster
                        # than everything else anyway
    print "(End Native)"
    
    i = 0
    j = 0
    imax = len(set1)
    jmax = len(set2)
    while i < imax and j < jmax:
        if set1[i] == set2[j]:
            retset.add(set1[i])
            i += 1
            j += 1
        elif set1[i] < set2[j]:
            i += 1
        else:
            j += 1
    return retset


s1 = range(23,9720)
print len(s1)
s2 = range(28,20970,23)
print len(s2)

from time import time


#Where n = len(s1) and m = len(s2)
# naiveIntersect should be O(n*m)
# binarysearchIntersect should be O(n*log(m)) (plus C overhead for sorting....O(m*log(m)))
# minIntersect should be O(n+m) (plus C overhead for sorting O(m*log(m)+n*log(n)))



print "Naive..."
start = time()
ret1 = naiveIntersect(s1, s2)       #W
print ".........OK",(time() - start), "seconds"
print "Binary Search..."
start = time()
ret2 = binarysearchIntersect(s1, s2)
print "................OK",(time() - start), "seconds"
print "Min Search..."
start = time()
ret3 = minIntersect(s1, s2)
print ".............OK",(time() - start), "seconds"

print ret1 == ret2 == ret3